package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.commons.io.FileUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serial;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class CityGmlArchive {

    private static final Logger logger = LogManager.getLogger(CityGmlArchive.class);

    @Serial
    private static final long serialVersionUID = 2168389511043362615L;

    private List<CityGmlZipEntry> entries;

    private boolean validated = false;

    public static CityGmlArchive fromZipFile(String zipFile, ParserConfiguration config) {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        Path tmpDir = null;
        try (ZipFile zipFileObj = new ZipFile(zipFile)) {
            tmpDir = Files.createTempDirectory("zipTmp");
            tmpDir.toFile().deleteOnExit();
            for (Enumeration<? extends ZipEntry> entries = zipFileObj.entries(); entries.hasMoreElements();) {
                ZipEntry entry = entries.nextElement();
                if (entry.isDirectory()) {
                    continue;
                }
                if (entry.getName().endsWith(".gml")) {
                    InputStream is = zipFileObj.getInputStream(entry);
                    Files.copy(is, Path.of( tmpDir.toString(),entry.getName()));
                    archiveEntries.add(CityGmlZipEntry.of(entry, config, zipFile, tmpDir));
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (Exception e){
            throw e;
        } finally {
            if (tmpDir != null) {
                try {
                    FileUtils.deleteDirectory(tmpDir.toFile());
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
        logger.info("Parsed zip-archive with {} entries", archiveEntries.size());
        return new CityGmlArchive(archiveEntries);
    }

    private CityGmlArchive(List<CityGmlZipEntry> entries) {
        this.entries = entries;
    }

    public void exportToZipFile(String path)  {
        if (!path.endsWith(".zip")){
            throw new IllegalArgumentException("Target zip-filepath must end with '.zip'");
        }


        if (validated) {
            //TODO: Export of validation reports for each entry
        }
    }

    public void checkEntries(){
        validated = false;
        for(CityGmlZipEntry entry : entries){
            Checker checker = new Checker(entry.getModel());
            checker.runChecks();
        }
        validated = true;
    }

    public List<CityGmlZipEntry> getEntries() {
        return entries;
    }
}
