package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.zip.ZipEntry;

public class CityGmlZipEntry {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);


    private String fileName;
    private String archivePath;
    private CityDoctorModel model;

    private boolean erroneousEntry = false;
    private boolean isLibraryObject = false;
    private boolean validated = false;

    public static CityGmlZipEntry of(ZipEntry entry, ParserConfiguration config, String archivePath, Path tempDir){
        CityGmlZipEntry cgzEntry = new CityGmlZipEntry();
        cgzEntry.fileName = entry.getName();
        cgzEntry.archivePath = archivePath;
        try {
            CityGmlParser.gagLogger(true);
            cgzEntry.model = CityGmlParser.parseCityGmlFile(tempDir.toString() +"\\" +entry.getName(), config);
        }
        catch (CityGmlParseException | InvalidGmlFileException e) {
            logger.error(e);
            cgzEntry.erroneousEntry = true;
        }
        return cgzEntry;
    }

    public void validateModel(){
        if(erroneousEntry){
            return;
        }
        Checker checker = new Checker(this.getModel());
        checker.runChecks();
        validated = true;
    }

    public boolean isValidated(){
        return validated;
    }

    public void setIsLibraryObject(boolean isLibraryObject) {
        this.isLibraryObject = isLibraryObject;
    }

    public String getFileName() {
        return fileName;
    }

    public String getArchivePath() {
        return archivePath;
    }

    public CityDoctorModel getModel() {
        return model;
    }

    public boolean isLibraryObject() {
        return isLibraryObject;
    }

    public boolean isErroneousEntry() {
        return erroneousEntry;
    }
}
